// MODEL 203 DATA ACQUISITON SYSTEM - DLL HEADER
// Lawson Labs, Inc.
// 3239 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800
// Last modified 11-13-00


// Sets 4-byte aligment on all structures declared within this header
#pragma pack(push, 4)

#include <windows.h>
//*************************************************************************//
//  M203 Defines
//
#define RANGE				1.2 		 // Input voltage Range factor 2 for +/- 5Vdc
											 // units, 1.2 for Gas Chromatography units
#define PICCRYSTAL		19.6608   // M203 board crystal frequency in Mhz
#define MAXCOUNT			0x3FFF	 // Maximum DAC resolution (2^14) 

//*************************************************************************//
//  M203 VARIABLES
//
//  		This structure defines variables used to pass data to and from 
//		the Model 203 DLL functions.
//
typedef struct{
	short		comPort;			// Currently selected comm port (0 to 3) for COM1 .. COM4
	double	scanInterval;	// Current scan interval in seconds
	short		numChannels;	// Number of scanning channels (always 1 for Model 203)
	short		tScan;			// Scan type (always 0 for Model 203)
	short		scanChan;		// Current scan mode channel (0,1,6, or 7)
	short    lastDigInp;		// Last digital input value
	short 	version;			// Model 203 version (203)
	short 	pollChan;			// Current polled mode channel
	short 	subSample;		// Number of subsamples per conversion 160 for 60HZ, 192 for 50 Hz
	unsigned long rawCount;	// Last raw count returned from Model 203
	int 	   digOutWord;		// Last value sent to Model 203 for digital output (0-255)
	short		wordCnt;			// number of bytes in a conversion result (2 = 16-bit, 3 = 24-bit)
	int 	   errorType;		// error code returned from DLL
	int		bU;				// BiPolar UniPolar Input Range (Not Used on Model 203)
	short 	filter;			// Input filter selection (not used on Model 203)      
	short 	divideBy;		// Remote (power of 2) division (0 to 7)
	double 	gain;				// Last calculated Gain
	float 	maxRate;			// Maximum sample rate (120 Hz for 60Hz, 100 Hz for 50Hz)
	short 	totalPeriods;	// (modeRegLo) - Number of conversions per sample
	short 	modeRegLo;		//
	short 	modeRegMid;		// MSB = Word Length, always 1 (24 bit), remaining bits unused 
	short 	modeRegHi;		// Mode Register High - Unused on Model 203
	short		baud;				// BAUD rate code (0-5) 0=9600 bps ... 5=300 bps
	float		dac1volts;		// Current Voltage for Analog Output 1
	float		dac2volts;    	// Current Voltage for Analog Output 2
	short		dacEnable;    	// Not Used (DACs always enabled)
	int		timeBase;  	  	// must be a positive value
	long 		echoTestCount; // Serial Echo Test Byte Count
	unsigned long 	offsetTotal;	// Last calculated Offset
	unsigned long 	runningAvg;		// Running Average
	short		dataAcqMode;	// 0 = Scanning, 1 = Polled
	unsigned int inputBuffer;	//receive queue sizes for Windows COMM driver
	short		wL;          	// Word Length - Always 128 for 24-bit with M203
	float	 	gain1;         // not used!!!!!!!!!!!!!
	double	totalBits;		// Total possible bits of resolution
	BYTE		scan;        	//
	BYTE		scan1;        	//
	BYTE		scan2;       	//
	int  		retry;			// Number of SignOn retrys
   short		ChanChgFlg;
	double	Voltage;         	// Last calculated voltage
}	T_M203;
T_M203 *M203;

#define FALSE 0
#define TRUE 1
#define DEV	FALSE
#define DBG_LOW 1
#define DBG_HIGH 2
int BufferSize=0;				//  current size of buffer
COMSTAT portStat;	  			// structure for comm status
BYTE DoDebug = 0;
char* strDebugMsg;
//
//*************************************************************************//
//  LOCAL INTERNAL DEFINES
//
#define CLOCK_RATE (1092)	//  clock rate in (clock cycles per minute)
#define CHANNELS 4			//  number of channels used by caller
#define COM1  0				// COM port numbering
#define COM2  1
#define COM3  2
#define COM4  3
#define TXQUEUESIZE 64		//  transmit queue size as used by Windows
COMMTIMEOUTS cto;

//*************************************************************************//
//  GLOBAL VARIABLES
//
double 		samplePeriod; 	
float 		scan;
float			scan1;
float			scan2;
float 		secRoll;
float			rollScan; 
short		  	triggerFlg=0;
unsigned long offsetTotal=0;
unsigned	long FullScaleTotal=0;
// Also Global is the Model 203 Variable Structure defined in M203DLL.h
//  device channel to com port as used by Windows
#define INVALIDCOMDEV INVALID_HANDLE_VALUE
static HANDLE idComDev[16]={INVALIDCOMDEV,   //  device channel to com port as used by Windows
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV,
									 INVALIDCOMDEV};
DWORD  lpdwErrors;			// address of variable to receive error codes

DCB dcb;  				//com port control structure used by Windows
int ComHandleNum = 1;		//default to com port 1 in array
double MinTime;
double MaxTime;
double BaudTime;
double ConvTime;
double WaitTime;
double CharWait;
int ScanWords = 3;
BYTE last_char;
BYTE *bptr;
BYTE cbuf[25];
int sType=1;   					// 0 = single-channel scans, 1 = normal scans
int sToken;
int bsize;
int version;						//
static BYTE RunningCheckSum;	//
int clearRemote = 0;				//
int numDigInputs = 0, numScans = 0;	 // counters for debugging
BYTE firstTime=0;
int firstInQue;
int GlobalGain;
int totalPeriods = 1; 			// Total number of conversions included in one sample
long offset;	 
long total;
double calGain = 1;	 
long count;				 
int runningAvg = 4;
long tmv = 0;			 
int aptr, chanChg;	 
int aDutyCycle;
clock_t dacTime;	  				// timer for updating dacs while not running
DWORD comErr;						// address of variable to receive error codes

BYTE *pNewScanBuff;
//  channel description
//  the entries marked (*) are only used in acs[0]
//*************************************************************************//
//  LOCAL FUNCTION PROTOTYPES
//		The following are local routines
//
static int 	send_byte(BYTE c);				 // Sends one byte to DAS
static int 	receive_byte(BYTE *bptr);	 // receives one byte from DAS and updates checksum	
static int	send_packet(int Outb1,int Outb2);  // Sends a 2-byte packet and a packet checksum to the DAS
static int 	wait_char(int chars,float milliseconds);  // Receives XX bytes within YY seconds
void    		delay(float milliseconds);  //  delay for given number of milliseconds
void			flush_port(void);            // removes all bytes from comm port input que
void  		get_port_status(void);        // Checks status and unlocks comm port
void  		request_digital_input(void);            // Requests digital input from Model 203
void  		set_dacs(int c);  				 // Sets the voltage of either analog outputs						
void 			calc_vars(); // Calculates system variables
void 			round_it(double *Interval);  // Rounds the selected scan interval
//6-21-06 short GetVoltage(void);
double   CountArray[100];  // Running voltage average array
short		ArrayPointer;   	// Running voltage average array pointer
short GetSubConversion(void);

//*************************************************************************//
//  LOCAL FUNCTION PROTOTYPES
//		The following are local routines
/*
	__declspec( dllexport )  short  ClearVoltageArray(void);
	__declspec( dllexport )  short  SetPtrs(T_M203 *M203, BYTE*);
	__declspec( dllexport )  short  SignOn(short EchoTestFlg, BYTE fDebug);
	__declspec( dllexport )  short  FullScaleCal(void);
	__declspec( dllexport )  short  OffsetCal(void);
	__declspec( dllexport )  short  SysCal(void);
	__declspec( dllexport )  short  GetDigIn(int token);
	__declspec( dllexport )  void  SetDigOut(void);
   __declspec( dllexport )  short  GetVer(void);
   __declspec( dllexport )  short  GetConversion(void);
   __declspec( dllexport )  void  SetChanCode(void);
   __declspec( dllexport )  short  StartScanning(void);
   __declspec( dllexport )  short  GetOneScan(void);
   __declspec( dllexport )  short  StopScanning(void);
   __declspec( dllexport )  short  SendADMode(void);
   __declspec( dllexport )  short  GetCheckSum(void);
   __declspec( dllexport )  void  CloseDevice(void);
	__declspec( dllexport )  short  ChangeAnalogOutput(short DacToSet);
   __declspec( dllexport )  short  CheckScanTime(double *maxTime, double *minTime);
	__declspec( dllexport )  short  EchoTest(void);
	__declspec( dllexport )  void   Gconv(void);
   __declspec( dllexport )  void   RemFromBuffer(short size);
*/

extern "C"
{

	//5-25-06
	__declspec( dllexport )  UINT  GetNumScansInBuffer(void);
	__declspec( dllexport )  void  Gconv(void);

	//6-21-06
	__declspec( dllexport )  short  GetVoltage(void);


	__declspec( dllexport )  short  ClearVoltageArray(void);


//*************************************************************************//
//   	These DLL functions provide access to the Model 203 - Data
//	Acquisition System.

//**************************************************************************
//  SETPTRS - This function sets any pointers to be used between the DLL
//			and instance of calling application. Especially important, is it
//			sets the pointer to the instance of the M203 structured data.
//       Note that the memory for the structure is established in the calling
//			calling application, and the DLL holds only a pointer to that memory
//
// Other Routines Called: None
// Output: None
// Return Values: BOOLEAN
//		TRUE = Success.
//		FALSE = Failed.
	__declspec( dllexport )  short  
		SetPtrs(T_M203 *M203, BYTE* pAppNewScanBuff, char* strDebugMsg);

//**************************************************************************
//  SIGNON - This function establishes communications with the Model 203
//		using the settings in M203 configuration file and the
//		selected BAUD rate, comm Port, and input buffer size.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->baud, M203->comPort, M203->modeRegHi, M203->modeRegMid,
//			M203->totalPeriods, M203->timeBase, M203->divideBy, M203->scanInterval
//			M203->scan, M203->scan1, M203->scan2, M203->inputBuffer
//		EchoTestFlag - 1 for SignOn with EchoTest, 0 for normal SignOn
//			Note: If EchoTest is used, two calls to SignOn must be performed.
//			The first call executes the first portion of the signon procedure and
//			starts the serial byte echo test.  The second call stops the echo test
//			and completes the sign on procedure.
//
// Other Routines Called: None
// Output:
//		M203->retry, M203->echoTestCount
// Return Values:
//		0 = Successful sign on, port opened, Model 203 active.
//		1 = Unsuccessful signon, comm port is closed
//
	__declspec(dllexport)  short  SignOn(short EchoTestFlg, BYTE fDebug);
//==========================================================================

//**************************************************************************
// FULL-SCALE CALIBRATION - Perform a full-scale calibration on the currently
//		selected channel, makes the currently selected channel read 5 volts.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->runningAvg, M203->offsetTotal, M203->rawCount, RANGE
//
// Other Routines Called: GetConversion
// Output: M203->FullScaleTotal, M203->gain, M203->totalBits
// Return Values:
// 	0 = Successful Full-Scale Calibration
//		1 = Count difference between the full-scale reading and the
//				last offset calibration reading is too small to produce
//				an acceptable calibration
//		2 = GetConversion Error
	__declspec( dllexport )  short   FullScaleCal(void);
//==========================================================================

//**************************************************************************
// OFFSET CALIBRATION - Perform an offset calibration on the currently
//		selected channel.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->runningAvg, M203->rawCount
// Other Routines Called: GetConversion
// Output: M203->offsetTotal
// Return Values:
//  	0 = Succesful Offset calibration
//	 	1 = GetConversion Error
	__declspec( dllexport )  short   OffsetCal(void);
//==========================================================================

//**************************************************************************
// SYSCAL - Perform a system calibration which consists of an offset calibration
//		on channel 7 followed by a full-scale calibration on channel 6. Once
//		calibration is completed, the active channel is reselected.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->scanChan, M203->pollChan
// Other Routines Called: OffsetCal, FullScaleCal
// Output:
// Return Values:
//		0 = No errors, a successfull system calibration was performed
//   	1 = Calibration error
	__declspec( dllexport )  short   SysCal(void);
//==========================================================================

//**************************************************************************
// GET DIGITAL INPUT - This routine instructs the Model 203 to read the
//		"RS" digital input.
//
// Input: token = Get Digital Input Command token
//			80 = Get digital input, 8E = Get digital input with checksum
// 		Argument = always 4Ch
// Other Routines Called: None
// Output: 	Bit 0 of M203->lastDigInp = 0 if "RS" input is low (0 Vdc),
//										        = 1 if "RS" input is high (5 Vdc)
// Return Values:
//		0 = Successfull input read
//	 	1 = Incorrect running checksum (8E digital checksum only)
//	 	2 = Incorrect GetDigitalInput command token echoed
//    3 = TimeOut waiting for command echo
	__declspec( dllexport )  short   GetDigIn(int token);
//==========================================================================

//**************************************************************************
// CHANGE DIGITAL OUTPUT - This function instructs the Model 203 to output
//		the digital output code in M203->digOutWord to the digital output
//		port termials "A" through "H".
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->digOutWord
// Other Routines Called: None
// Output:
// Return Values: none
//
	__declspec( dllexport )  void   SetDigOut(void);
//==========================================================================

//**************************************************************************
// Get Version - This function obtains the current version number from
//		the data acquisition system and places it into the M203 Variable
//		sturcture as M203->version.
//
// Input: none
// Other Routines Called: None
// Output: M203->version
// Return Values:
//   	0 = No Errors
//		1 = Timeout waiting for Version Number to be returned
//		2 = Incorrect command token echoed
//		3 = Timeout waiting for command token to be echoed
   __declspec( dllexport )  short   GetVer(void);
//==========================================================================

//**************************************************************************
// GET CONVERSION - This routine gets one conversion data sample from the
//		Model 203 on the selected channel. The conversion result is placed
//		into M203->rawCount.  This function is used to obtain data samples
//		during polled mode and to perform calibrations.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->wordCnt
// Other Routines Called: None
// Output: M203->rawCount
// Return Values:
//   	0 = No Errors, new conversion obtained
//		1 = Incorrect get conversion token echoed
// 	2 = TimeOut waiting for GetConversion command token and data
//				OR communications port error
// 	3 = Error in sending conversion request
   __declspec( dllexport )  short   GetConversion(void);
//==========================================================================

//**************************************************************************
// SET CHANNEL CODE - This function instructs the Model 203 to select the
//		desired channel.  If scanning (M203->dataAcqMode = 0), M203->scanChan
//		is selected.  If polled mode (M203->dataAcqMode = 1), MM203->pollChan
//		is selected.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->dataAcqMode, M203->scanChan, M203->pollChan
// Other routines called: none
// Output: none
// Return Values: None
   __declspec( dllexport )  void   SetChanCode(void);
//==========================================================================

//**************************************************************************
//	START SCANNING - This function instructs the Model 203 to start scanning
//		using the selected scanning mode.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->wordCnt, M203->scanInterval
//	  ScanToken		0x8B = Normal Scanning
//						0x8D = CheckSum Scanning
//						0x89 = Normal, Triggered Scanning
//						0x8C = Checksum, Triggered Scanning
//
//		Edge (required for triggered scanning only)
//							0 = Positive Edge Trigger
//               		!0 = Negative Edge Trigger
//
// Other Routines Called: None
// Output: None
// Return Values:
//		0 = Successfull scanning start
//   	1 = Incorrect command token echoed.
//   	2 = Time out waiting for command token echo.
   __declspec( dllexport )  short   StartScanning(int ScanToken, int Edge);
//==========================================================================

//**************************************************************************
//	GET ONE SCAN - This function gets one scan from the input buffer. The
//		result is placed in M203->rawCount.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->inputBuffer, M203->scanInterval, M203->wordCnt
// Other Routines Called: None
// Output: M203->rawCount
// Return Values:
//   -1 = Time Out occurred waiting for scan (waiting for Trigger)
// 	0 = Scan received.
//   	1 = Time Out occurred waiting for scan
//   	2 = Input Buffer overflow detected.
   __declspec( dllexport )  short   GetOneScan(void);
//==========================================================================

//**************************************************************************
//	STOP SCANNING - This function instructs the Model 203 to stop scanning and
//		empties the input buffer.
//
// Input: None
// Other Routines Called: None
// Output: None
// Return Values:
//		0 = Successfull, Scanning is discontinued.
//	 	1 = Stop Scan command token not echoed.
   __declspec( dllexport )  short   StopScanning(void);
//==========================================================================

//**************************************************************************
// SEND ADMODE - Sets the Model 203 mode registers.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->modeRegHi, M203->modeRegLo, M203->modeRegLo,
//			M203->totalPeriods, M203->wL, M203->timeBase
// Other Routines Called: None
// Output:
// Return Values:
//		0 = Successfull, no Errors
//   	1 = mode registers echoed are incorrect
//		2 = Time out waiting for moderegisters to be echoed
//		3 = Incorrect command token echoed
//		4 = Time out waiting for command token echo
   __declspec( dllexport )  short   SendADMode(void);
//==========================================================================

//**************************************************************************
// GET CHECKSUM - This function obtains and tests the Model 203 running
//		checksum.
//
// Other Routines Called: WaitChar, SendPacket
// Output - Global Variables: None
// 	RunningCheckSum = 0 - Anytime this routine is called, the running
//			checksum is set to zero
// Output - Return Values:
//   	0 = CheckSum OK, No Errors
//		1 = Checksum received is incorrect
//		2 = GetCheckSum Token echoed incorrectly
//		3 = TimeOut while waiting for checksum command and running checksum
//				byte to be echoed
   __declspec( dllexport )  short   GetCheckSum(void);
//==========================================================================

//**************************************************************************
//	CLOSE DEVICE - This function sends the stop scan command to stop a scan
//  if scanning, sends a master reset command, and closes the currently
//  selected comm port.
//
// Input: None
// Other Routines Called: None
// Output: None
// Return Values: None
   __declspec( dllexport )  void   CloseDevice(void);
//==========================================================================

//********************************************************************/
// CHANGE ANALOG OUTPUT - This function instructs the Model 203 to change
//		the output voltage of the selected analog output.
//
// Input:
//		NewVoltage = New voltage for analog output - 0 to 5 Volts
//		DacToSet = 1 or 2
// Global Variables Referenced: MAXCOUNT
// Other Routines Called:
// Output - Global Variables: None
//		M203->dac1volts = voltage of analog output 1 OR
//		M203->dac2volts = voltage of analog output 2
// Output - Return Values:
//   	0 = No Errors -
//    1 = Invalid Analog Output Port
	__declspec( dllexport )  short   ChangeAnalogOutput(short DacToSet);
//==========================================================================

//**************************************************************************
// CheckScanTime - This function changes the Model 203 scan interval. If the
//		new scan interval is within acceptable limits (mintime < scanInterval <
//		maxtime), M203->ScanInterval is updated with NewScanInterval. 	Also,
//		MinTime and MaxTime are updated based current baud rate, wordCnt,
//		etc.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->totalPeriods, M203->wordCnt, M203->baud
//		NewScanInterval = the requested scan interval
//
// Other Routines Called: None
// Output: Maxtime, Mintime, M203->scanInterval
// Return Values:
// 	0 = The selected Scan Interval is acceptable
//	 	1 = Scan interval NOT within limits
   __declspec( dllexport )  short   CheckScanTime(double *NewScanInterval,
   						double *maxTime, double *minTime);
//==========================================================================

//**************************************************************************
// ECHO TEST - This routine performs a serial byte echo test which can be used
//		to test serial communications.  This routines sends characters 1 through
//		through 255 (not 0x00) to the Model 203 and tests the eched bytes.
//		for each successful byte transferred, received, and compared, M203->
//		echoTestCount is incremented.
//
// Input:
// Other Routines Called: None
// Output:
// Return Values:
//   	0 = Successfull byte echo, no errors
//		1 = Wrong byte echoed
//		2 = Echo Test Timeout (waiting for echo byte)
	__declspec( dllexport )  short   EchoTest(void);
//==========================================================================

//TIM NOTE: may not need?
	__declspec( dllexport )  void    Gconv(void);
   __declspec( dllexport )  void    RemFromBuffer(short size);

//TIM NOTE: 203 only
}
/* ALL FUNCTIONS
	__declspec( dllexport )  short  ClearVoltageArray(void);
	__declspec( dllexport )  short  SetPtrs(T_M203 *M203, BYTE*);
	__declspec( dllexport )  short  SignOn(short EchoTestFlg, BYTE fDebug);
	__declspec( dllexport )  short  FullScaleCal(void);
	__declspec( dllexport )  short  OffsetCal(void);
	__declspec( dllexport )  short  SysCal(void);
	__declspec( dllexport )  short  GetDigIn(int token);
	__declspec( dllexport )  void  SetDigOut(void);
   __declspec( dllexport )  short  GetVer(void);
   __declspec( dllexport )  short  GetConversion(void);
   __declspec( dllexport )  void  SetChanCode(void);
   __declspec( dllexport )  short  StartScanning(void);
   __declspec( dllexport )  short  GetOneScan(void);
   __declspec( dllexport )  short  StopScanning(void);
   __declspec( dllexport )  short  SendADMode(void);
   __declspec( dllexport )  short  GetCheckSum(void);
   __declspec( dllexport )  void  CloseDevice(void);
	__declspec( dllexport )  short  ChangeAnalogOutput(short DacToSet);
   __declspec( dllexport )  short  CheckScanTime(double *maxTime, double *minTime);
	__declspec( dllexport )  short  EchoTest(void);
	__declspec( dllexport )  void   Gconv(void);
   __declspec( dllexport )  void   RemFromBuffer(short size);
*/
// use def. aligment
#pragma pack(pop)
